/// <reference path="libs/js/action.js" />
/// <reference path="libs/js/stream-deck.js" />

class OmnibusConnection {
	constructor(port, plugin){
		this.port = port;
		this.plugin = plugin;
		this.connect = async function (){
			try{
				const macosPlatforms = ['Macintosh', 'MacIntel', 'MacPPC', 'Mac68K'];
				if (macosPlatforms.indexOf(navigator.platform) !== -1) {
					const pathArr = location.pathname.split("/");
					const homedir = pathArr.slice(0, 3).join("/");
					const path = homedir + "/Library/Application Support/com.audiomovers.omnibus.driver/streamdeck.txt";
                    let request = new XMLHttpRequest();
                    request.open('GET', path);
                    request.onreadystatechange = function() {
                        if (request.readyState == 4 && (request.status === 200 || request.status == 0)) {
                    		let content = request.responseText;
							if (content.length > 0){
								const obj = JSON.parse(content);
								const new_port = obj.streamdeck_port;
								if (new_port !== undefined){
									this.port = new_port;
								}
							}
                        }
                    }
                    request.send();
				}
			} catch (err) {
				console.error("Error getting the websocket port", err);
			}

			try{
				this.websocket = new WebSocket('ws://127.0.0.1:' + this.port);
			} catch (err) {
				console.error("Error connecting to the Omnibus service", err);
			}
			this.websocket.onopen = () => {
				if (plugin)
					plugin.on_service_connect();
			};
	
			this.websocket.onerror = (evt) => {
				if (this.websocket)
					this.websocket.close();
			};
	
			this.websocket.onclose = (evt) => {
				if (this.websocket)
					this.websocket.close();
				if (plugin)
					plugin.on_service_disconnect();
				setTimeout(() => {
					this.connect();
				  }, 1000);
			};
	
			this.websocket.onmessage = (evt) => {
				if (plugin)
					plugin.on_service_message(evt.data);
			}
		}

		this.connect();
	}

	send(message){
		try{
			if (this.websocket)
				this.websocket.send(JSON.stringify(message));
		} catch (err) {
			console.error("Error sending a message to the Omnibus service", err);
		}
	}
}

class OmnibusPlugin {
	constructor(connection){
		this.streamdeck_connection = connection;
		this.omnibus_connection = new OmnibusConnection(23543, this);
	}

	on_service_connect(){

		const json = {
			action: "get",
			property: "all"
		};
		this.send_to_omnibus_service(json)
	}

	on_service_disconnect(){

	}

	on_service_message(message){
		const js_msg = JSON.parse(message);

		if (this.sample_rate_action){
			let sample_rate_value = js_msg["sample_rate"];
			if (sample_rate_value !== undefined)
				this.sample_rate_action.display_sample_rate(sample_rate_value);
		}

		if (this.buffer_size_action){
			let buffer_size_value = js_msg["buffer_size"];
			if (buffer_size_value !== undefined)
				this.buffer_size_action.display_buffer_size(buffer_size_value);
		}

		if (this.misc_action)
		{
			let input_lock = js_msg["input_volume_locked"];
			if (input_lock !== undefined)
				this.misc_action.display_input_lock(input_lock);

			let output_lock = js_msg["output_volume_locked"];
			if (output_lock !== undefined)
				this.misc_action.display_output_lock(output_lock);

			let use_colors = js_msg["use_colors_for_devices"];
			if (use_colors !== undefined)
				this.misc_action.display_use_colors(use_colors);
	
			let matrix_visible = js_msg["matrix_visible"];
			if (matrix_visible !== undefined)
				this.misc_action.display_matrix_visible(matrix_visible);

			let always_on_top = js_msg["always_on_top"];
			if (always_on_top !== undefined)
				this.misc_action.display_always_on_top(always_on_top);
		}

		if (this.scale_action){
			let scale_ratio_value = js_msg["scale_ratio"];
			if (scale_ratio_value !== undefined)
				this.scale_action.display_scale_ratio(scale_ratio_value);
		}

		if (this.snapshots_action){
			let active_snapshot_value = js_msg["active_snapshot"];
			if (active_snapshot_value !== undefined)
				this.snapshots_action.display_current_snapshot(active_snapshot_value);

			let active_snapshot_name = js_msg["active_snapshot_name"];
			if (active_snapshot_name !== undefined)
				this.snapshots_action.display_current_snapshot_name(active_snapshot_name);
		}
	}

	setTitle(context, title){
		if (this.streamdeck_connection){
			let msg = {
				"title": title,
				"target": DestinationEnum.HARDWARE_AND_SOFTWARE
			};

			let json = {
				"event": "setTitle",
				"context": context,
				"payload": msg
			};
			this.streamdeck_connection.send(JSON.stringify(json));	
		}
	}

	setImage(context, image){
		$SD.setImage(context, image);
	}

	async setFileImage(context, filename){
		let image_text = "data:image/svg+xml;charset=utf8,";
		try {
			const path = "actions/omnibus/assets/" + filename;
            let request = new XMLHttpRequest();
            request.open('GET', path);
            request.onreadystatechange = function() {
                if (request.readyState == 4 && (request.status === 200 || request.status == 0)) {
					let content = request.responseText;
					if (content.length > 0) {
                    	$SD.setImage(context, "data:image/svg+xml;charset=utf8," + content);
					}
                }
            }
            request.send();
		} catch (err) {
			console.error(err);
		}
	}

	async setFileImageAndText(context, filename, text_value){
		let image_text = "data:image/svg+xml;charset=utf8,";
		try {
			const path = "actions/omnibus/assets/" + filename;
            let request = new XMLHttpRequest();
            request.open('GET', path);
            request.onreadystatechange = function() {
                if (request.readyState == 4 && (request.status === 200 || request.status == 0)) {
                    let content = request.responseText;
					if (content.length > 0) {
						content = content.replace("</svg>", text_value + "</svg>");
						$SD.setImage(context, "data:image/svg+xml;charset=utf8," + content);
					}
                }
            }
            request.send();
		} catch (err) {
			console.error(err);
		}
	}

	setSampleRateAction(action){
		this.sample_rate_action = action;
	}

	setBufferSizeAction(action){
		this.buffer_size_action = action;
	}

	setMiscAction(action){
		this.misc_action = action;
	}

	setScaleAction(action){
		this.scale_action = action;
	}

	setSnapshotsAction(action){
		this.snapshots_action = action;
	}

	send_to_omnibus_service(message){
		if (this.omnibus_connection)
			this.omnibus_connection.send(message);
	}
}

class OmnibusBufferSizeAction extends Action {
	constructor(plugin){
		super('com.audiomovers.omnibus.streamdeck.buffer-size-action');

		this.action_types = [];
		this.plugin = plugin;
		this.onWillAppear(({ action, context, device, event, payload }) => {
			const obj = JSON.parse(JSON.stringify(payload.settings));
			let action_type = obj.action_type;
			if (action_type === undefined)
				action_type = "current";
						
			this.action_types[context] = action_type;

			if (action_type == "current"){
				const json = {
					action: "get",
					property: "buffer_size"
				};
				this.plugin.send_to_omnibus_service(json)
			} else {
				this.display_buffer_size_to_set(action_type);
			}
		});

		this.onDidReceiveSettings(({context, payload}) => {
			const obj = JSON.parse(JSON.stringify(payload.settings));
			let action_type = obj.action_type;
			if (action_type === undefined)
				action_type = "current";

			this.action_types[context] = action_type;

			if (action_type == "current"){
				const json = {
					action: "get",
					property: "buffer_size"
				};
				this.plugin.send_to_omnibus_service(json)
			} else {
				this.display_buffer_size_to_set(action_type);
			}
		});

		this.onKeyUp(({ action, context, device, event, payload }) => {
			let action_type = this.action_types[context];
			if (action_type === undefined)
				action_type = "current";
			if (action_type != "current")
			{
				const json = {
					action: "set",
					property: "buffer_size",
					value: action_type
				};
				this.plugin.send_to_omnibus_service(json)
			}
		});
		
		this.onKeyDown(({ action, context, device, event, payload }) => {

		});
	
		this.onSendToPlugin(({context, payload}) => {
		});
	}

	display_buffer_size(buffer_size_value)
	{
		if (this.plugin){
			if (buffer_size_value){
				for (const [context, action_type] of Object.entries(this.action_types)) {
					if (action_type == "current"){
						plugin.setFileImage(context, "buffer_size/display/" + buffer_size_value + ".svg");
					}
				}
			}
		}
	}

	display_buffer_size_to_set(buffer_size_value){
		if (this.plugin){
			if (buffer_size_value){
				for (const [context, action_type] of Object.entries(this.action_types)) {
					if (action_type === buffer_size_value){
						plugin.setFileImage(context, "buffer_size/set/" + buffer_size_value + ".svg");
					}
				}
			}
		}
	}
}

class OmnibusSampleRateAction extends Action {
	constructor(plugin){
		super('com.audiomovers.omnibus.streamdeck.sample-rate-action');
		this.plugin = plugin;
		this.action_types = [];
		this.onWillAppear(({ action, context, device, event, payload }) => {
			
			const obj = JSON.parse(JSON.stringify(payload.settings));
			let action_type = obj.action_type;
			if (action_type === undefined)
				action_type = "current";
						
			this.action_types[context] = action_type;

			if (action_type == "current"){
				const json = {
					action: "get",
					property: "sample_rate"
				};
				this.plugin.send_to_omnibus_service(json)
			} else {
				this.display_sample_rate_to_set(action_type);
			}
		});

		this.onDidReceiveSettings(({context, payload}) => {
			const obj = JSON.parse(JSON.stringify(payload.settings));
			let action_type = obj.action_type;
			if (action_type === undefined)
				action_type = "current";

			this.action_types[context] = action_type;

			if (action_type == "current"){
				const json = {
					action: "get",
					property: "sample_rate"
				};
				this.plugin.send_to_omnibus_service(json)
			} else {
				this.display_sample_rate_to_set(action_type);
			}
		});

		this.onKeyUp(({ action, context, device, event, payload }) => {
			let action_type = this.action_types[context];
			if (action_type === undefined)
				action_type = "current";

			if (action_type != "current")
			{
				const json = {
					action: "set",
					property: "sample_rate",
					value: action_type
				};
				this.plugin.send_to_omnibus_service(json)
			}

		});
		
		this.onKeyDown(({ action, context, device, event, payload }) => {

		});
	
		this.onSendToPlugin(({context, payload}) => {

		});
	}

	display_sample_rate(sample_rate_value)
	{
		if (this.plugin){
			if (sample_rate_value){
				for (const [context, action_type] of Object.entries(this.action_types)) {
					if (action_type == "current")
					{
						switch(sample_rate_value){
							case "44100":
								plugin.setFileImage(context, "sample_rate/display/44.1k.svg");
								break;
							case "48000":
								plugin.setFileImage(context, "sample_rate/display/48k.svg");
								break;
							case "88200":
								plugin.setFileImage(context, "sample_rate/display/88.2k.svg");
								break;
							case "96000":
								plugin.setFileImage(context, "sample_rate/display/96k.svg");
								break;
							case "176400":
								plugin.setFileImage(context, "sample_rate/display/176.4k.svg");
								break;
							case "192000":
								plugin.setFileImage(context, "sample_rate/display/192k.svg");
								break;																	
							default:
								break;
						}
					}
				}
			}
		}
	}

	display_sample_rate_to_set(sample_rate_value){
		if (this.plugin){
			if (sample_rate_value){
				for (const [context, action_type] of Object.entries(this.action_types)) {
					if (action_type === sample_rate_value)
					{
						switch(sample_rate_value){
							case "44100":
								plugin.setFileImage(context, "sample_rate/set/44.1k.svg");
								break;
							case "48000":
								plugin.setFileImage(context, "sample_rate/set/48k.svg");
								break;
							case "88200":
								plugin.setFileImage(context, "sample_rate/set/88.2k.svg");
								break;
							case "96000":
								plugin.setFileImage(context, "sample_rate/set/96k.svg");
								break;
							case "176400":
								plugin.setFileImage(context, "sample_rate/set/176.4k.svg");
								break;
							case "192000":
								plugin.setFileImage(context, "sample_rate/set/192k.svg");
								break;																	
							default:
								break;
						}
					}
				}
			}
		}
	}
}

class OmnibusMiscAction extends Action {
	constructor(plugin){
		super('com.audiomovers.omnibus.streamdeck.misc-action');
		this.plugin = plugin;
		this.action_types = [];
		this.onWillAppear(({ action, context, device, event, payload }) => {
			
			const obj = JSON.parse(JSON.stringify(payload.settings));
			let action_type = obj.action_type;
			if (action_type === undefined)
				action_type = "input_volume_locked";
						
			if (action_type !== undefined){
				this.action_types[context] = action_type;

					const json = {
						action: "get",
						property: action_type
					};
					this.plugin.send_to_omnibus_service(json)
			}
	
		});

		this.onDidReceiveSettings(({context, payload}) => {
			const obj = JSON.parse(JSON.stringify(payload.settings));
			let action_type = obj.action_type;
			if (action_type !== undefined){
				this.action_types[context] = action_type;

					const json = {
						action: "get",
						property: action_type
					};
					this.plugin.send_to_omnibus_service(json)
			}
		});

		this.onKeyUp(({ action, context, device, event, payload }) => {
			let action_type = this.action_types[context];
			if (action_type !== undefined)
			{
				const json = {
					action: "set",
					property: action_type
				};
				this.plugin.send_to_omnibus_service(json)
			}
		});
		
		this.onKeyDown(({ action, context, device, event, payload }) => {

		});
	
		this.onSendToPlugin(({context, payload}) => {

		});
	}

	display_input_lock(lock_value)
	{
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type == "input_volume_locked")
				{
					if (lock_value)
						plugin.setFileImage(context, "misc/input_locked.svg");
					else
						plugin.setFileImage(context, "misc/input_unlocked.svg");
				}
			}
		}
	}

	display_output_lock(lock_value)
	{
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type == "output_volume_locked")
				{
					if (lock_value)
						plugin.setFileImage(context, "misc/output_locked.svg");
					else
						plugin.setFileImage(context, "misc/output_unlocked.svg");
				}
			}
		}
	}

	display_use_colors(use_colors)
	{
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type == "use_colors_for_devices")
				{
					if (use_colors)
						plugin.setFileImage(context, "misc/use_color_active.svg");
					else
						plugin.setFileImage(context, "misc/use_color_inactive.svg");
				}
			}
		}
	}

	display_matrix_visible(matrix_visible)
	{
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type == "matrix_visible")
				{
					if (matrix_visible)
						plugin.setFileImage(context, "misc/routing_matrix_active.svg");
					else
						plugin.setFileImage(context, "misc/routing_matrix_inactive.svg");
				}
			}
		}
	}

	display_always_on_top(always_on_top)
	{
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type == "always_on_top")
				{
					if (always_on_top)
						plugin.setFileImage(context, "misc/always_on_top_active.svg");
					else
						plugin.setFileImage(context, "misc/always_on_top_inactive.svg");
				}
			}
		}
	}
}

class OmnibusScaleAction extends Action {
	constructor(plugin){
		super('com.audiomovers.omnibus.streamdeck.scale-action');

		this.action_types = [];
		this.plugin = plugin;
		this.onWillAppear(({ action, context, device, event, payload }) => {
			const obj = JSON.parse(JSON.stringify(payload.settings));
			let action_type = obj.action_type;
			if (action_type === undefined)
				action_type = "current";
						
			this.action_types[context] = action_type;

			if (action_type == "current"){
				const json = {
					action: "get",
					property: "scale_ratio"
				};
				this.plugin.send_to_omnibus_service(json)
			} else {
				this.display_scale_ratio_to_set(action_type);
			}
		});

		this.onDidReceiveSettings(({context, payload}) => {
			const obj = JSON.parse(JSON.stringify(payload.settings));
			let action_type = obj.action_type;
			if (action_type === undefined)
				action_type = "current";

			this.action_types[context] = action_type;

			if (action_type == "current"){
				const json = {
					action: "get",
					property: "scale_ratio"
				};
				this.plugin.send_to_omnibus_service(json)
			} else {
				this.display_scale_ratio_to_set(action_type);
			}
		});

		this.onKeyUp(({ action, context, device, event, payload }) => {
			let action_type = this.action_types[context];
			if (action_type === undefined)
				action_type = "current";
			if (action_type != "current")
			{
				const json = {
					action: "set",
					property: "scale_ratio",
					value: action_type
				};
				this.plugin.send_to_omnibus_service(json)
			}
		});
		
		this.onKeyDown(({ action, context, device, event, payload }) => {

		});
	
		this.onSendToPlugin(({context, payload}) => {
		});
	}

	display_scale_ratio(scale_ratio)
	{
		if (this.plugin){
			if (scale_ratio){
				for (const [context, action_type] of Object.entries(this.action_types)) {
					if (action_type == "current"){
						plugin.setFileImage(context, "scale/display/" + scale_ratio + ".svg");
					}
				}
			}
		}
	}

	display_scale_ratio_to_set(scale_ratio){
		if (this.plugin){
			if (scale_ratio){
				for (const [context, action_type] of Object.entries(this.action_types)) {
					if (action_type === scale_ratio){
						plugin.setFileImage(context, "scale/set/" + scale_ratio + ".svg");
					}
				}
			}
		}
	}
}

class OmnibusSnapshotsAction extends Action {
	constructor(plugin){
		super('com.audiomovers.omnibus.streamdeck.snapshots-action');
		this.plugin = plugin;
		this.action_types = [];
		this.onWillAppear(({ action, context, device, event, payload }) => {
			const obj = JSON.parse(JSON.stringify(payload.settings));
			let action_type = obj.action_type;
			if (action_type === undefined)
				action_type = "current";

			if (action_type !== undefined)
				this.action_types[context] = action_type;

				switch(action_type){
					case "current":
					{
						const json = {
							action: "get",
							property: "active_snapshot"
						};
						this.plugin.send_to_omnibus_service(json)
					}
						break;
					case "current_name":
					{
						const json = {
							action: "get",
							property: "active_snapshot_name"
						};
						this.plugin.send_to_omnibus_service(json)
					}
						break;
					default:
						this.display_snapshot_action(action_type);
						break;
				}
		});

		this.onDidReceiveSettings(({context, payload}) => {
			const obj = JSON.parse(JSON.stringify(payload.settings));
			let action_type = obj.action_type;
			if (action_type !== undefined){
				this.action_types[context] = action_type;

				switch(action_type){
					case "current":
					{
						const json = {
							action: "get",
							property: "active_snapshot"
						};
						this.plugin.send_to_omnibus_service(json)
					}
						break;
					case "current_name":
					{
						const json = {
							action: "get",
							property: "active_snapshot_name"
						};
						this.plugin.send_to_omnibus_service(json)
					}
						break;
					default:
						this.display_snapshot_action(action_type);
						break;
				}
			}
		});

		this.onKeyUp(({ action, context, device, event, payload }) => {
			let action_type = this.action_types[context];
			if (action_type !== undefined)
			{
				if (action_type.startsWith("recall_snapshot_")){
					let snapshot_number = parseInt(action_type.replace("recall_snapshot_", ""));
					const json = {
						action: "set",
						property: "active_snapshot",
						value: snapshot_number.toString()
					};
					this.plugin.send_to_omnibus_service(json)
				} else {
					const json = {
						action: "set",
						property: action_type
					};
					this.plugin.send_to_omnibus_service(json)
				}
			}
		});
		
		this.onKeyDown(({ action, context, device, event, payload }) => {

		});
	
		this.onSendToPlugin(({context, payload}) => {

		});
	}

	display_current_snapshot(active_snapshot_value){
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type == "current"){
					let display_value = "None";
					if (active_snapshot_value >= 0)
						display_value = parseInt(active_snapshot_value) + 1;
					let svg_text = "<text fill='#000' font-family='Monospace' font-size='13' x='37.5' y='48' dominant-baseline='middle' text-anchor='middle'>Current:</text>";
					svg_text += "<text fill='#000' font-family='Monospace' font-size='13' x='37.5' y='65' dominant-baseline='middle' text-anchor='middle'>" + display_value + "</text>";
					plugin.setFileImageAndText(context, "snapshots/current_template.svg", svg_text);
				}
			}
		}
	}

	display_current_snapshot_name(name){
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type == "current_name"){
					let display_value = "None";
					if (name != "")
						display_value = name;
					let svg_text = "<text fill='#FFF' font-family='Monospace' font-size='12' x='37.5' y='50' dominant-baseline='middle' text-anchor='middle'>" + display_value + "</text>";
					plugin.setFileImageAndText(context, "snapshots/name_template.svg", svg_text);
				}
			}
		}
	}

	display_next_snapshot()
	{
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type == "next_snapshot")
					plugin.setFileImage(context, "snapshots/next.svg");
			}
		}
	}

	display_prev_snapshot()
	{
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type == "prev_snapshot")
					plugin.setFileImage(context, "snapshots/prev.svg");
			}
		}
	}

	display_save_snapshot()
	{
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type == "save_snapshot")
					plugin.setFileImage(context, "snapshots/save.svg");
			}
		}
	}

	display_delete_snapshot()
	{
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type == "delete_snapshot")
					plugin.setFileImage(context, "snapshots/delete.svg");
			}
		}
	}

	display_snapshot_to_recall(action_type)
	{
		if (this.plugin){
			for (const [context, action_type] of Object.entries(this.action_types)) {
				if (action_type.startsWith("recall_snapshot_")){
					let snapshot_number = parseInt(action_type.replace("recall_snapshot_", ""));					
					let display_value = "Recall: " + (snapshot_number + 1);
					let svg_text = "<text fill='#000' font-family='Monospace' font-size='13' x='37.5' y='60' dominant-baseline='middle' text-anchor='middle'>" + display_value + "</text>";
					plugin.setFileImageAndText(context, "snapshots/current_template.svg", svg_text);
				}
			}
		}
	}

	display_snapshot_action(action_type){
		switch(action_type){
			case "next_snapshot":
				this.display_next_snapshot();
				break;
			case "prev_snapshot":
				this.display_prev_snapshot();
				break;
			case "save_snapshot":
				this.display_save_snapshot();
				break;
			case "delete_snapshot":
				this.display_delete_snapshot();
				break;					
			default:
				if (action_type.startsWith("recall_snapshot_"))
					this.display_snapshot_to_recall(action_type);
				break;
		}
	}
}

let plugin;
let buffer_size_action;
let sample_rate_action;
let misc_action;
let scale_action;
let snapshots_action;

$SD.onConnected(({ actionInfo, appInfo, connection, messageType, port, uuid }) => {
	con = connection;
	plugin = new OmnibusPlugin(connection);
	buffer_size_action = new OmnibusBufferSizeAction(plugin);
	sample_rate_action = new OmnibusSampleRateAction(plugin);
	misc_action = new OmnibusMiscAction(plugin);
	scale_action = new OmnibusScaleAction(plugin);
	snapshots_action = new OmnibusSnapshotsAction(plugin);

	plugin.setSampleRateAction(sample_rate_action);
	plugin.setBufferSizeAction(buffer_size_action);
	plugin.setMiscAction(misc_action);
	plugin.setScaleAction(scale_action);
	plugin.setSnapshotsAction(snapshots_action);
});
